from datetime import datetime
from threading import RLock
from typing import Any, Dict, List, Optional, Protocol, Tuple, Union

import jwt

import accelbyte_py_sdk.api.iam as iam_service
import accelbyte_py_sdk.api.iam.models as iam_models
import accelbyte_py_sdk.services.auth as auth_service
from accelbyte_py_sdk import AccelByteSDK
from accelbyte_py_sdk.core import Timer

from ._bloom_filter import BloomFilter
from ._ctypes import PermissionAction, create_permission_struct
from ._ctypes import FetchValidationDataError, FetchRoleError
from ._ctypes import (
    TokenValidationError,
    InsufficientPermissionsError,
    TokenRevokedError,
    UserRevokedError,
)
from ._ctypes import PermissionAction, PermissionStruct, create_permission_struct
from ._utils import replace_resource, validate_permission

PublicPrivateKey = Any
JWKSet = Dict[str, PublicPrivateKey]
JWTClaims = Dict[str, Any]
Role = iam_models.ModelRoleResponseV3
NamespaceRole = Dict[str, str]


def str2datetime(s: str) -> datetime:
    # from: 'YYYY-mm-ddTHH:MM:SS.fffffffffZ'
    # to:   'YYYY-mm-ddTHH:MM:SS.fffZ+0000'
    tz = "Z+0000" if s.endswith("Z") else ""  # Add explicit UTC timezone.
    s = s[0:23] + tz
    return datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.%fZ%z")


class JWKSCache(Timer):
    JWKS_KEYS_KEY: str = "keys"

    def __init__(self, sdk: AccelByteSDK, interval: Optional[Union[int, float]] = None):
        self.sdk = sdk
        self._jwks: Dict[str, PublicPrivateKey] = {}
        self._lock = RLock()
        if interval is not None:
            Timer.__init__(
                self,
                interval,
                self.update,
                daemon=True,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
            )

    def update(self):
        result, error = iam_service.get_jwksv3(sdk=self.sdk)
        if error:
            return

        with self._lock:
            keys = result.to_dict().get(JWKSCache.JWKS_KEYS_KEY, [])
            jwks = jwt.PyJWKSet(keys)
            for jwk in jwks.keys:
                self._jwks[jwk.key_id] = jwk.key

    def get_key_from_cache(self, key_id: str) -> Optional[PublicPrivateKey]:
        with self._lock:
            return self._jwks.get(key_id, None)

    def get_key(self, key_id: str) -> Optional[PublicPrivateKey]:
        key = self.get_key_from_cache(key_id)
        if key is not None:
            return key

        self.update()

        return self.get_key_from_cache(key_id)


class RevocationListCache(Timer):
    def __init__(self, sdk: AccelByteSDK, interval: Union[int, float]):
        self.sdk = sdk
        self._revoked_token_filter: Optional[BloomFilter] = None
        self._revoked_users: Dict[str, float] = {}
        self._lock = RLock()
        Timer.__init__(
            self,
            interval,
            self.update,
            daemon=True,
            repeats=-1,
            autostart=True,
            repeat_on_exception=True,
        )

    def update(self):
        result, error = iam_service.get_revocation_list_v3(sdk=self.sdk)
        if error:
            return

        with self._lock:
            # Revoked Tokens
            revoked_tokens = result.revoked_tokens
            self._revoked_token_filter = BloomFilter.create_from_bits(
                bits=revoked_tokens.bits, k=revoked_tokens.k, m=revoked_tokens.m
            )
            # Revoked Users
            revoked_users = result.revoked_users
            self._revoked_users = {}
            for user in revoked_users:
                if user.id_ and user.revoked_at:
                    revoked_at = str2datetime(user.revoked_at).timestamp()
                    self._revoked_users[user.id_] = revoked_at

    def is_token_revoked(self, token: str) -> bool:
        with self._lock:
            return self._revoked_token_filter.might_contains(key=token)

    def is_user_revoked(self, user_id: str, issued_at: int) -> bool:
        with self._lock:
            revoked_at = self._revoked_users.get(user_id)
            if revoked_at:
                return revoked_at >= issued_at
        return False


class RolesCache(Timer):
    def __init__(self, sdk: AccelByteSDK, interval: Union[int, float]):
        self.sdk = sdk
        self._roles: Dict[str, Any] = {}
        self._lock = RLock()
        Timer.__init__(
            self,
            interval,
            self.clear,
            daemon=True,
            repeats=-1,
            autostart=True,
            repeat_on_exception=True,
        )

    def clear(self):
        with self._lock:
            self._roles = {}

    def update(self):
        pass

    def cache_role(self, role_id: str) -> Any:
        role, error = iam_service.admin_get_role_v3(role_id=role_id, sdk=self.sdk)
        if error:
            return error
        with self._lock:
            self._roles[role_id] = role
        return None

    def get_role(self, role_id: str) -> Optional[Role]:
        with self._lock:
            role = self._roles.get(role_id, None)
            if role:
                return role

        error = self.cache_role(role_id=role_id)
        if error:
            raise FetchRoleError(f"role ID: {role_id}")

        return self._roles.get(role_id)

    def get_role_permissions(self, role_id: str) -> List[PermissionStruct]:
        role = self.get_role(role_id=role_id)
        return role.permissions

    def get_modified_role_permissions(
        self, role_id: str, namespace: str, user_id: Optional[str] = None
    ) -> List[PermissionStruct]:
        permissions = self.get_role_permissions(role_id=role_id)
        return [
            create_permission_struct(
                action=p.action,
                resource=replace_resource(
                    resource=p.resource,
                    namespace=namespace,
                    user_id=user_id,
                ),
            )
            for p in permissions
        ]

    def get_modified_role_permissions2(
        self, namespace_role: NamespaceRole, user_id: Optional[str] = None
    ) -> List[PermissionStruct]:
        return self.get_modified_role_permissions(
            role_id=namespace_role.get("roleId"),
            namespace=namespace_role.get("namespace"),
            user_id=user_id,
        )


class CachingTokenValidator:
    DEFAULT_DECODE_ALGORITHMS: List[str] = ["RS256"]
    DEFAULT_DECODE_OPTIONS: Dict[str, Any] = {"verify_aud": False, "verify_exp": True}
    JWS_HEADER_PARAM_KEY_ID_KEY: str = "kid"

    def __init__(
        self,
        sdk: AccelByteSDK,
        algorithms: Optional[List[str]] = None,
        options: Optional[Dict[str, Any]] = None,
        publisher_namespace: Optional[str] = None,
        token_refresh_interval: Optional[Union[int, float]] = None,
        jwks_refresh_interval: Optional[Union[int, float]] = None,
        revocation_list_refresh_interval: Union[int, float] = 60,
        role_cache_time: Union[int, float] = 60,
    ) -> None:
        self.algorithms = (
            algorithms
            if algorithms is not None
            else CachingTokenValidator.DEFAULT_DECODE_ALGORITHMS
        )
        self.options = (
            options
            if options is not None
            else CachingTokenValidator.DEFAULT_DECODE_OPTIONS
        )
        self.publisher_namespace = publisher_namespace

        if token_refresh_interval is not None:
            auth_service.LoginClientTimer(
                token_refresh_interval,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
                sdk=sdk,
            )

        self.jwks_cache = JWKSCache(sdk, jwks_refresh_interval)
        self.revocation_list_cache = RevocationListCache(
            sdk, revocation_list_refresh_interval
        )
        self.roles_cache = RolesCache(sdk, role_cache_time)

        self.jwks_cache.update()
        self.revocation_list_cache.update()
        self.roles_cache.update()

    def has_valid_permissions(
        self,
        claims: JWTClaims,
        permission: PermissionStruct,
        namespace: Optional[str] = None,
    ) -> bool:
        claims_namespace = claims.get("namespace", None)
        user_id = claims.get("user_id", None)

        target = create_permission_struct(
            action=permission.action,
            resource=replace_resource(
                resource=permission.resource,
                namespace=namespace,
                token_namespace=claims_namespace,
                publisher_namespace=self.publisher_namespace,
                user_id=user_id,
            ),
        )

        # Check claims.permissions
        claims_permissions = [
            create_permission_struct(action=p.get("Action"), resource=p.get("Resource"))
            for p in claims.get("permissions", [])
        ]
        if claims_permissions and validate_permission(
            target=target,
            permissions=claims_permissions,
        ):
            return True

        # Check claim.namespace_roles
        claims_namespace_roles = claims.get("namespace_roles", [])
        if user_id and claims_namespace_roles:
            nr_permissions = []
            for nr in claims_namespace_roles:
                nr_permissions.extend(
                    self.roles_cache.get_modified_role_permissions2(
                        namespace_role=nr, user_id=user_id
                    )
                )
            if nr_permissions and validate_permission(
                target=target,
                permissions=nr_permissions,
            ):
                return True

        # Check claim.roles
        claims_roles = claims.get("roles", [])
        if claims_roles:
            r_permissions = []
            for r in claims_roles:
                r_permissions.extend(
                    self.roles_cache.get_modified_role_permissions(
                        role_id=r, namespace=namespace, user_id=user_id
                    )
                )
            if r_permissions and validate_permission(
                target=target,
                permissions=r_permissions,
            ):
                return True

        return False

    def validate_token(
        self,
        token: str,
        resource: Optional[str] = None,
        action: Optional[PermissionAction] = None,
        namespace: Optional[str] = None,
        **kwargs,
    ) -> Optional[Exception]:
        # Check if token was revoked.
        if self.revocation_list_cache.is_token_revoked(token=token):
            return TokenRevokedError()

        claims, error = self._decode(token=token, **kwargs)
        if error:
            return error

        # Check if user was revoked.
        if claims_user_id := claims.get("user_id"):
            if self.revocation_list_cache.is_user_revoked(
                user_id=claims_user_id, issued_at=claims.get("iat")
            ):
                return UserRevokedError()

        # Check if the claims has valid permissions.
        if (
            resource is not None
            and action is not None
            and not self.has_valid_permissions(
                claims=claims,
                permission=create_permission_struct(action, resource),
                namespace=namespace,
            )
        ):
            return InsufficientPermissionsError()

        return None

    def _decode(
        self,
        token: str,
        algorithms: Optional[List[str]] = None,
        options: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> Tuple[Optional[JWTClaims], Optional[Exception]]:
        algorithms = algorithms if algorithms is not None else self.algorithms
        options = options if options is not None else self.options

        header_params = jwt.get_unverified_header(jwt=token)
        if not (
            kid := header_params.get(CachingTokenValidator.JWS_HEADER_PARAM_KEY_ID_KEY)
        ):
            return None, KeyError(CachingTokenValidator.JWS_HEADER_PARAM_KEY_ID_KEY)

        if not (key := self.jwks_cache.get_key(kid)):
            return None, KeyError(kid)

        claims = jwt.decode(
            jwt=token,
            key=key,
            algorithms=algorithms,
            options=options,
        )

        if "user_id" not in claims and (sub := claims.get("sub")):
            claims["user_id"] = sub

        return claims, None


__all__ = [
    "CachingTokenValidator",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Social Service (2.8.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class CycleTypeEnum(StrEnum):
    ANNUALLY = "ANNUALLY"
    DAILY = "DAILY"
    MONTHLY = "MONTHLY"
    SEASONAL = "SEASONAL"
    WEEKLY = "WEEKLY"


class StatusEnum(StrEnum):
    ACTIVE = "ACTIVE"
    INIT = "INIT"
    STOPPED = "STOPPED"


class StatCycleInfo(Model):
    """Stat cycle info (StatCycleInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        current_version: (currentVersion) REQUIRED int

        cycle_type: (cycleType) REQUIRED Union[str, CycleTypeEnum]

        id_: (id) REQUIRED str

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        reset_time: (resetTime) REQUIRED str

        start: (start) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        updated_at: (updatedAt) REQUIRED str

        description: (description) OPTIONAL str

        end: (end) OPTIONAL str

        next_reset: (nextReset) OPTIONAL str

        reset_date: (resetDate) OPTIONAL int

        reset_day: (resetDay) OPTIONAL int

        reset_month: (resetMonth) OPTIONAL int

        season_period: (seasonPeriod) OPTIONAL int
    """

    # region fields

    created_at: str  # REQUIRED
    current_version: int  # REQUIRED
    cycle_type: Union[str, CycleTypeEnum]  # REQUIRED
    id_: str  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    reset_time: str  # REQUIRED
    start: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    updated_at: str  # REQUIRED
    description: str  # OPTIONAL
    end: str  # OPTIONAL
    next_reset: str  # OPTIONAL
    reset_date: int  # OPTIONAL
    reset_day: int  # OPTIONAL
    reset_month: int  # OPTIONAL
    season_period: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> StatCycleInfo:
        self.created_at = value
        return self

    def with_current_version(self, value: int) -> StatCycleInfo:
        self.current_version = value
        return self

    def with_cycle_type(self, value: Union[str, CycleTypeEnum]) -> StatCycleInfo:
        self.cycle_type = value
        return self

    def with_id(self, value: str) -> StatCycleInfo:
        self.id_ = value
        return self

    def with_name(self, value: str) -> StatCycleInfo:
        self.name = value
        return self

    def with_namespace(self, value: str) -> StatCycleInfo:
        self.namespace = value
        return self

    def with_reset_time(self, value: str) -> StatCycleInfo:
        self.reset_time = value
        return self

    def with_start(self, value: str) -> StatCycleInfo:
        self.start = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> StatCycleInfo:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> StatCycleInfo:
        self.updated_at = value
        return self

    def with_description(self, value: str) -> StatCycleInfo:
        self.description = value
        return self

    def with_end(self, value: str) -> StatCycleInfo:
        self.end = value
        return self

    def with_next_reset(self, value: str) -> StatCycleInfo:
        self.next_reset = value
        return self

    def with_reset_date(self, value: int) -> StatCycleInfo:
        self.reset_date = value
        return self

    def with_reset_day(self, value: int) -> StatCycleInfo:
        self.reset_day = value
        return self

    def with_reset_month(self, value: int) -> StatCycleInfo:
        self.reset_month = value
        return self

    def with_season_period(self, value: int) -> StatCycleInfo:
        self.season_period = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "current_version"):
            result["currentVersion"] = int(self.current_version)
        elif include_empty:
            result["currentVersion"] = 0
        if hasattr(self, "cycle_type"):
            result["cycleType"] = str(self.cycle_type)
        elif include_empty:
            result["cycleType"] = Union[str, CycleTypeEnum]()
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "reset_time"):
            result["resetTime"] = str(self.reset_time)
        elif include_empty:
            result["resetTime"] = ""
        if hasattr(self, "start"):
            result["start"] = str(self.start)
        elif include_empty:
            result["start"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "end"):
            result["end"] = str(self.end)
        elif include_empty:
            result["end"] = ""
        if hasattr(self, "next_reset"):
            result["nextReset"] = str(self.next_reset)
        elif include_empty:
            result["nextReset"] = ""
        if hasattr(self, "reset_date"):
            result["resetDate"] = int(self.reset_date)
        elif include_empty:
            result["resetDate"] = 0
        if hasattr(self, "reset_day"):
            result["resetDay"] = int(self.reset_day)
        elif include_empty:
            result["resetDay"] = 0
        if hasattr(self, "reset_month"):
            result["resetMonth"] = int(self.reset_month)
        elif include_empty:
            result["resetMonth"] = 0
        if hasattr(self, "season_period"):
            result["seasonPeriod"] = int(self.season_period)
        elif include_empty:
            result["seasonPeriod"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        current_version: int,
        cycle_type: Union[str, CycleTypeEnum],
        id_: str,
        name: str,
        namespace: str,
        reset_time: str,
        start: str,
        status: Union[str, StatusEnum],
        updated_at: str,
        description: Optional[str] = None,
        end: Optional[str] = None,
        next_reset: Optional[str] = None,
        reset_date: Optional[int] = None,
        reset_day: Optional[int] = None,
        reset_month: Optional[int] = None,
        season_period: Optional[int] = None,
        **kwargs,
    ) -> StatCycleInfo:
        instance = cls()
        instance.created_at = created_at
        instance.current_version = current_version
        instance.cycle_type = cycle_type
        instance.id_ = id_
        instance.name = name
        instance.namespace = namespace
        instance.reset_time = reset_time
        instance.start = start
        instance.status = status
        instance.updated_at = updated_at
        if description is not None:
            instance.description = description
        if end is not None:
            instance.end = end
        if next_reset is not None:
            instance.next_reset = next_reset
        if reset_date is not None:
            instance.reset_date = reset_date
        if reset_day is not None:
            instance.reset_day = reset_day
        if reset_month is not None:
            instance.reset_month = reset_month
        if season_period is not None:
            instance.season_period = season_period
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> StatCycleInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "currentVersion" in dict_ and dict_["currentVersion"] is not None:
            instance.current_version = int(dict_["currentVersion"])
        elif include_empty:
            instance.current_version = 0
        if "cycleType" in dict_ and dict_["cycleType"] is not None:
            instance.cycle_type = str(dict_["cycleType"])
        elif include_empty:
            instance.cycle_type = Union[str, CycleTypeEnum]()
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "resetTime" in dict_ and dict_["resetTime"] is not None:
            instance.reset_time = str(dict_["resetTime"])
        elif include_empty:
            instance.reset_time = ""
        if "start" in dict_ and dict_["start"] is not None:
            instance.start = str(dict_["start"])
        elif include_empty:
            instance.start = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "end" in dict_ and dict_["end"] is not None:
            instance.end = str(dict_["end"])
        elif include_empty:
            instance.end = ""
        if "nextReset" in dict_ and dict_["nextReset"] is not None:
            instance.next_reset = str(dict_["nextReset"])
        elif include_empty:
            instance.next_reset = ""
        if "resetDate" in dict_ and dict_["resetDate"] is not None:
            instance.reset_date = int(dict_["resetDate"])
        elif include_empty:
            instance.reset_date = 0
        if "resetDay" in dict_ and dict_["resetDay"] is not None:
            instance.reset_day = int(dict_["resetDay"])
        elif include_empty:
            instance.reset_day = 0
        if "resetMonth" in dict_ and dict_["resetMonth"] is not None:
            instance.reset_month = int(dict_["resetMonth"])
        elif include_empty:
            instance.reset_month = 0
        if "seasonPeriod" in dict_ and dict_["seasonPeriod"] is not None:
            instance.season_period = int(dict_["seasonPeriod"])
        elif include_empty:
            instance.season_period = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, StatCycleInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[StatCycleInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[StatCycleInfo, List[StatCycleInfo], Dict[Any, StatCycleInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "currentVersion": "current_version",
            "cycleType": "cycle_type",
            "id": "id_",
            "name": "name",
            "namespace": "namespace",
            "resetTime": "reset_time",
            "start": "start",
            "status": "status",
            "updatedAt": "updated_at",
            "description": "description",
            "end": "end",
            "nextReset": "next_reset",
            "resetDate": "reset_date",
            "resetDay": "reset_day",
            "resetMonth": "reset_month",
            "seasonPeriod": "season_period",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "currentVersion": True,
            "cycleType": True,
            "id": True,
            "name": True,
            "namespace": True,
            "resetTime": True,
            "start": True,
            "status": True,
            "updatedAt": True,
            "description": False,
            "end": False,
            "nextReset": False,
            "resetDate": False,
            "resetDay": False,
            "resetMonth": False,
            "seasonPeriod": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "cycleType": ["ANNUALLY", "DAILY", "MONTHLY", "SEASONAL", "WEEKLY"],
            "status": ["ACTIVE", "INIT", "STOPPED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Matchmaking Service (2.25.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_mock_ticket import ModelsMockTicket


class ModelsGetMockTicketsResponse(Model):
    """Models get mock tickets response (models.GetMockTicketsResponse)

    Properties:
        cancelled_count: (cancelled_count) REQUIRED int

        highest_mmr: (highest_mmr) REQUIRED int

        in_queue_count: (in_queue_count) REQUIRED int

        lowest_mmr: (lowest_mmr) REQUIRED int

        matched_count: (matched_count) REQUIRED int

        mmr_in_queue: (mmr_in_queue) REQUIRED Dict[str, int]

        mmr_matched: (mmr_matched) REQUIRED Dict[str, int]

        mmr_timeout: (mmr_timeout) REQUIRED Dict[str, int]

        tickets: (tickets) REQUIRED List[ModelsMockTicket]

        timeout_count: (timeout_count) REQUIRED int
    """

    # region fields

    cancelled_count: int  # REQUIRED
    highest_mmr: int  # REQUIRED
    in_queue_count: int  # REQUIRED
    lowest_mmr: int  # REQUIRED
    matched_count: int  # REQUIRED
    mmr_in_queue: Dict[str, int]  # REQUIRED
    mmr_matched: Dict[str, int]  # REQUIRED
    mmr_timeout: Dict[str, int]  # REQUIRED
    tickets: List[ModelsMockTicket]  # REQUIRED
    timeout_count: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_cancelled_count(self, value: int) -> ModelsGetMockTicketsResponse:
        self.cancelled_count = value
        return self

    def with_highest_mmr(self, value: int) -> ModelsGetMockTicketsResponse:
        self.highest_mmr = value
        return self

    def with_in_queue_count(self, value: int) -> ModelsGetMockTicketsResponse:
        self.in_queue_count = value
        return self

    def with_lowest_mmr(self, value: int) -> ModelsGetMockTicketsResponse:
        self.lowest_mmr = value
        return self

    def with_matched_count(self, value: int) -> ModelsGetMockTicketsResponse:
        self.matched_count = value
        return self

    def with_mmr_in_queue(self, value: Dict[str, int]) -> ModelsGetMockTicketsResponse:
        self.mmr_in_queue = value
        return self

    def with_mmr_matched(self, value: Dict[str, int]) -> ModelsGetMockTicketsResponse:
        self.mmr_matched = value
        return self

    def with_mmr_timeout(self, value: Dict[str, int]) -> ModelsGetMockTicketsResponse:
        self.mmr_timeout = value
        return self

    def with_tickets(
        self, value: List[ModelsMockTicket]
    ) -> ModelsGetMockTicketsResponse:
        self.tickets = value
        return self

    def with_timeout_count(self, value: int) -> ModelsGetMockTicketsResponse:
        self.timeout_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "cancelled_count"):
            result["cancelled_count"] = int(self.cancelled_count)
        elif include_empty:
            result["cancelled_count"] = 0
        if hasattr(self, "highest_mmr"):
            result["highest_mmr"] = int(self.highest_mmr)
        elif include_empty:
            result["highest_mmr"] = 0
        if hasattr(self, "in_queue_count"):
            result["in_queue_count"] = int(self.in_queue_count)
        elif include_empty:
            result["in_queue_count"] = 0
        if hasattr(self, "lowest_mmr"):
            result["lowest_mmr"] = int(self.lowest_mmr)
        elif include_empty:
            result["lowest_mmr"] = 0
        if hasattr(self, "matched_count"):
            result["matched_count"] = int(self.matched_count)
        elif include_empty:
            result["matched_count"] = 0
        if hasattr(self, "mmr_in_queue"):
            result["mmr_in_queue"] = {
                str(k0): int(v0) for k0, v0 in self.mmr_in_queue.items()
            }
        elif include_empty:
            result["mmr_in_queue"] = {}
        if hasattr(self, "mmr_matched"):
            result["mmr_matched"] = {
                str(k0): int(v0) for k0, v0 in self.mmr_matched.items()
            }
        elif include_empty:
            result["mmr_matched"] = {}
        if hasattr(self, "mmr_timeout"):
            result["mmr_timeout"] = {
                str(k0): int(v0) for k0, v0 in self.mmr_timeout.items()
            }
        elif include_empty:
            result["mmr_timeout"] = {}
        if hasattr(self, "tickets"):
            result["tickets"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.tickets
            ]
        elif include_empty:
            result["tickets"] = []
        if hasattr(self, "timeout_count"):
            result["timeout_count"] = int(self.timeout_count)
        elif include_empty:
            result["timeout_count"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        cancelled_count: int,
        highest_mmr: int,
        in_queue_count: int,
        lowest_mmr: int,
        matched_count: int,
        mmr_in_queue: Dict[str, int],
        mmr_matched: Dict[str, int],
        mmr_timeout: Dict[str, int],
        tickets: List[ModelsMockTicket],
        timeout_count: int,
        **kwargs,
    ) -> ModelsGetMockTicketsResponse:
        instance = cls()
        instance.cancelled_count = cancelled_count
        instance.highest_mmr = highest_mmr
        instance.in_queue_count = in_queue_count
        instance.lowest_mmr = lowest_mmr
        instance.matched_count = matched_count
        instance.mmr_in_queue = mmr_in_queue
        instance.mmr_matched = mmr_matched
        instance.mmr_timeout = mmr_timeout
        instance.tickets = tickets
        instance.timeout_count = timeout_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsGetMockTicketsResponse:
        instance = cls()
        if not dict_:
            return instance
        if "cancelled_count" in dict_ and dict_["cancelled_count"] is not None:
            instance.cancelled_count = int(dict_["cancelled_count"])
        elif include_empty:
            instance.cancelled_count = 0
        if "highest_mmr" in dict_ and dict_["highest_mmr"] is not None:
            instance.highest_mmr = int(dict_["highest_mmr"])
        elif include_empty:
            instance.highest_mmr = 0
        if "in_queue_count" in dict_ and dict_["in_queue_count"] is not None:
            instance.in_queue_count = int(dict_["in_queue_count"])
        elif include_empty:
            instance.in_queue_count = 0
        if "lowest_mmr" in dict_ and dict_["lowest_mmr"] is not None:
            instance.lowest_mmr = int(dict_["lowest_mmr"])
        elif include_empty:
            instance.lowest_mmr = 0
        if "matched_count" in dict_ and dict_["matched_count"] is not None:
            instance.matched_count = int(dict_["matched_count"])
        elif include_empty:
            instance.matched_count = 0
        if "mmr_in_queue" in dict_ and dict_["mmr_in_queue"] is not None:
            instance.mmr_in_queue = {
                str(k0): int(v0) for k0, v0 in dict_["mmr_in_queue"].items()
            }
        elif include_empty:
            instance.mmr_in_queue = {}
        if "mmr_matched" in dict_ and dict_["mmr_matched"] is not None:
            instance.mmr_matched = {
                str(k0): int(v0) for k0, v0 in dict_["mmr_matched"].items()
            }
        elif include_empty:
            instance.mmr_matched = {}
        if "mmr_timeout" in dict_ and dict_["mmr_timeout"] is not None:
            instance.mmr_timeout = {
                str(k0): int(v0) for k0, v0 in dict_["mmr_timeout"].items()
            }
        elif include_empty:
            instance.mmr_timeout = {}
        if "tickets" in dict_ and dict_["tickets"] is not None:
            instance.tickets = [
                ModelsMockTicket.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["tickets"]
            ]
        elif include_empty:
            instance.tickets = []
        if "timeout_count" in dict_ and dict_["timeout_count"] is not None:
            instance.timeout_count = int(dict_["timeout_count"])
        elif include_empty:
            instance.timeout_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsGetMockTicketsResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsGetMockTicketsResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsGetMockTicketsResponse,
        List[ModelsGetMockTicketsResponse],
        Dict[Any, ModelsGetMockTicketsResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "cancelled_count": "cancelled_count",
            "highest_mmr": "highest_mmr",
            "in_queue_count": "in_queue_count",
            "lowest_mmr": "lowest_mmr",
            "matched_count": "matched_count",
            "mmr_in_queue": "mmr_in_queue",
            "mmr_matched": "mmr_matched",
            "mmr_timeout": "mmr_timeout",
            "tickets": "tickets",
            "timeout_count": "timeout_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "cancelled_count": True,
            "highest_mmr": True,
            "in_queue_count": True,
            "lowest_mmr": True,
            "matched_count": True,
            "mmr_in_queue": True,
            "mmr_matched": True,
            "mmr_timeout": True,
            "tickets": True,
            "timeout_count": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Leaderboard Service (2.23.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsGetLeaderboardConfigRespV3(Model):
    """Models get leaderboard config resp V3 (models.GetLeaderboardConfigRespV3)

    Properties:
        all_time: (allTime) REQUIRED bool

        created_at: (createdAt) REQUIRED str

        cycle_ids: (cycleIds) REQUIRED List[str]

        descending: (descending) REQUIRED bool

        is_deleted: (isDeleted) REQUIRED bool

        leaderboard_code: (leaderboardCode) REQUIRED str

        name: (name) REQUIRED str

        stat_code: (statCode) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        deleted_at: (deletedAt) OPTIONAL str

        description: (description) OPTIONAL str

        icon_url: (iconURL) OPTIONAL str
    """

    # region fields

    all_time: bool  # REQUIRED
    created_at: str  # REQUIRED
    cycle_ids: List[str]  # REQUIRED
    descending: bool  # REQUIRED
    is_deleted: bool  # REQUIRED
    leaderboard_code: str  # REQUIRED
    name: str  # REQUIRED
    stat_code: str  # REQUIRED
    updated_at: str  # REQUIRED
    deleted_at: str  # OPTIONAL
    description: str  # OPTIONAL
    icon_url: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_all_time(self, value: bool) -> ModelsGetLeaderboardConfigRespV3:
        self.all_time = value
        return self

    def with_created_at(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.created_at = value
        return self

    def with_cycle_ids(self, value: List[str]) -> ModelsGetLeaderboardConfigRespV3:
        self.cycle_ids = value
        return self

    def with_descending(self, value: bool) -> ModelsGetLeaderboardConfigRespV3:
        self.descending = value
        return self

    def with_is_deleted(self, value: bool) -> ModelsGetLeaderboardConfigRespV3:
        self.is_deleted = value
        return self

    def with_leaderboard_code(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.leaderboard_code = value
        return self

    def with_name(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.name = value
        return self

    def with_stat_code(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.stat_code = value
        return self

    def with_updated_at(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.updated_at = value
        return self

    def with_deleted_at(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.deleted_at = value
        return self

    def with_description(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.description = value
        return self

    def with_icon_url(self, value: str) -> ModelsGetLeaderboardConfigRespV3:
        self.icon_url = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "all_time"):
            result["allTime"] = bool(self.all_time)
        elif include_empty:
            result["allTime"] = False
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "cycle_ids"):
            result["cycleIds"] = [str(i0) for i0 in self.cycle_ids]
        elif include_empty:
            result["cycleIds"] = []
        if hasattr(self, "descending"):
            result["descending"] = bool(self.descending)
        elif include_empty:
            result["descending"] = False
        if hasattr(self, "is_deleted"):
            result["isDeleted"] = bool(self.is_deleted)
        elif include_empty:
            result["isDeleted"] = False
        if hasattr(self, "leaderboard_code"):
            result["leaderboardCode"] = str(self.leaderboard_code)
        elif include_empty:
            result["leaderboardCode"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "stat_code"):
            result["statCode"] = str(self.stat_code)
        elif include_empty:
            result["statCode"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "deleted_at"):
            result["deletedAt"] = str(self.deleted_at)
        elif include_empty:
            result["deletedAt"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "icon_url"):
            result["iconURL"] = str(self.icon_url)
        elif include_empty:
            result["iconURL"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        all_time: bool,
        created_at: str,
        cycle_ids: List[str],
        descending: bool,
        is_deleted: bool,
        leaderboard_code: str,
        name: str,
        stat_code: str,
        updated_at: str,
        deleted_at: Optional[str] = None,
        description: Optional[str] = None,
        icon_url: Optional[str] = None,
        **kwargs,
    ) -> ModelsGetLeaderboardConfigRespV3:
        instance = cls()
        instance.all_time = all_time
        instance.created_at = created_at
        instance.cycle_ids = cycle_ids
        instance.descending = descending
        instance.is_deleted = is_deleted
        instance.leaderboard_code = leaderboard_code
        instance.name = name
        instance.stat_code = stat_code
        instance.updated_at = updated_at
        if deleted_at is not None:
            instance.deleted_at = deleted_at
        if description is not None:
            instance.description = description
        if icon_url is not None:
            instance.icon_url = icon_url
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsGetLeaderboardConfigRespV3:
        instance = cls()
        if not dict_:
            return instance
        if "allTime" in dict_ and dict_["allTime"] is not None:
            instance.all_time = bool(dict_["allTime"])
        elif include_empty:
            instance.all_time = False
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "cycleIds" in dict_ and dict_["cycleIds"] is not None:
            instance.cycle_ids = [str(i0) for i0 in dict_["cycleIds"]]
        elif include_empty:
            instance.cycle_ids = []
        if "descending" in dict_ and dict_["descending"] is not None:
            instance.descending = bool(dict_["descending"])
        elif include_empty:
            instance.descending = False
        if "isDeleted" in dict_ and dict_["isDeleted"] is not None:
            instance.is_deleted = bool(dict_["isDeleted"])
        elif include_empty:
            instance.is_deleted = False
        if "leaderboardCode" in dict_ and dict_["leaderboardCode"] is not None:
            instance.leaderboard_code = str(dict_["leaderboardCode"])
        elif include_empty:
            instance.leaderboard_code = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "statCode" in dict_ and dict_["statCode"] is not None:
            instance.stat_code = str(dict_["statCode"])
        elif include_empty:
            instance.stat_code = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "deletedAt" in dict_ and dict_["deletedAt"] is not None:
            instance.deleted_at = str(dict_["deletedAt"])
        elif include_empty:
            instance.deleted_at = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "iconURL" in dict_ and dict_["iconURL"] is not None:
            instance.icon_url = str(dict_["iconURL"])
        elif include_empty:
            instance.icon_url = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsGetLeaderboardConfigRespV3]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsGetLeaderboardConfigRespV3]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsGetLeaderboardConfigRespV3,
        List[ModelsGetLeaderboardConfigRespV3],
        Dict[Any, ModelsGetLeaderboardConfigRespV3],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "allTime": "all_time",
            "createdAt": "created_at",
            "cycleIds": "cycle_ids",
            "descending": "descending",
            "isDeleted": "is_deleted",
            "leaderboardCode": "leaderboard_code",
            "name": "name",
            "statCode": "stat_code",
            "updatedAt": "updated_at",
            "deletedAt": "deleted_at",
            "description": "description",
            "iconURL": "icon_url",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "allTime": True,
            "createdAt": True,
            "cycleIds": True,
            "descending": True,
            "isDeleted": True,
            "leaderboardCode": True,
            "name": True,
            "statCode": True,
            "updatedAt": True,
            "deletedAt": False,
            "description": False,
            "iconURL": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (6.0.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelVerificationCodeResponse(Model):
    """Model verification code response (model.VerificationCodeResponse)

    Properties:
        account_registration: (accountRegistration) REQUIRED str

        account_upgrade: (accountUpgrade) REQUIRED str

        password_reset: (passwordReset) REQUIRED str

        update_email: (updateEmail) REQUIRED str
    """

    # region fields

    account_registration: str  # REQUIRED
    account_upgrade: str  # REQUIRED
    password_reset: str  # REQUIRED
    update_email: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_account_registration(self, value: str) -> ModelVerificationCodeResponse:
        self.account_registration = value
        return self

    def with_account_upgrade(self, value: str) -> ModelVerificationCodeResponse:
        self.account_upgrade = value
        return self

    def with_password_reset(self, value: str) -> ModelVerificationCodeResponse:
        self.password_reset = value
        return self

    def with_update_email(self, value: str) -> ModelVerificationCodeResponse:
        self.update_email = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "account_registration"):
            result["accountRegistration"] = str(self.account_registration)
        elif include_empty:
            result["accountRegistration"] = ""
        if hasattr(self, "account_upgrade"):
            result["accountUpgrade"] = str(self.account_upgrade)
        elif include_empty:
            result["accountUpgrade"] = ""
        if hasattr(self, "password_reset"):
            result["passwordReset"] = str(self.password_reset)
        elif include_empty:
            result["passwordReset"] = ""
        if hasattr(self, "update_email"):
            result["updateEmail"] = str(self.update_email)
        elif include_empty:
            result["updateEmail"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        account_registration: str,
        account_upgrade: str,
        password_reset: str,
        update_email: str,
        **kwargs,
    ) -> ModelVerificationCodeResponse:
        instance = cls()
        instance.account_registration = account_registration
        instance.account_upgrade = account_upgrade
        instance.password_reset = password_reset
        instance.update_email = update_email
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelVerificationCodeResponse:
        instance = cls()
        if not dict_:
            return instance
        if "accountRegistration" in dict_ and dict_["accountRegistration"] is not None:
            instance.account_registration = str(dict_["accountRegistration"])
        elif include_empty:
            instance.account_registration = ""
        if "accountUpgrade" in dict_ and dict_["accountUpgrade"] is not None:
            instance.account_upgrade = str(dict_["accountUpgrade"])
        elif include_empty:
            instance.account_upgrade = ""
        if "passwordReset" in dict_ and dict_["passwordReset"] is not None:
            instance.password_reset = str(dict_["passwordReset"])
        elif include_empty:
            instance.password_reset = ""
        if "updateEmail" in dict_ and dict_["updateEmail"] is not None:
            instance.update_email = str(dict_["updateEmail"])
        elif include_empty:
            instance.update_email = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelVerificationCodeResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelVerificationCodeResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelVerificationCodeResponse,
        List[ModelVerificationCodeResponse],
        Dict[Any, ModelVerificationCodeResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "accountRegistration": "account_registration",
            "accountUpgrade": "account_upgrade",
            "passwordReset": "password_reset",
            "updateEmail": "update_email",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "accountRegistration": True,
            "accountUpgrade": True,
            "passwordReset": True,
            "updateEmail": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsCreateDeploymentOverrideRequest
from ..models import ModelsCreateDeploymentRequest
from ..models import ModelsCreateRegionOverrideRequest
from ..models import ModelsDeploymentWithOverride
from ..models import ModelsListDeploymentResponse
from ..models import ModelsUpdateDeploymentOverrideRequest
from ..models import ModelsUpdateDeploymentRequest
from ..models import ModelsUpdateRegionOverrideRequest
from ..models import ResponseError

from ..operations.deployment_config import CreateDeployment
from ..operations.deployment_config import CreateDeploymentClient
from ..operations.deployment_config import CreateDeploymentOverride
from ..operations.deployment_config import CreateOverrideRegionOverride
from ..operations.deployment_config import CreateRootRegionOverride
from ..operations.deployment_config import DeleteDeployment
from ..operations.deployment_config import DeleteDeploymentClient
from ..operations.deployment_config import DeleteDeploymentOverride
from ..operations.deployment_config import DeleteOverrideRegionOverride
from ..operations.deployment_config import DeleteRootRegionOverride
from ..operations.deployment_config import GetAllDeployment
from ..operations.deployment_config import GetAllDeploymentClient
from ..operations.deployment_config import GetDeployment
from ..operations.deployment_config import UpdateDeployment
from ..operations.deployment_config import UpdateDeploymentOverride
from ..operations.deployment_config import UpdateOverrideRegionOverride
from ..operations.deployment_config import UpdateRootRegionOverride


@same_doc_as(CreateDeployment)
def create_deployment(
    body: ModelsCreateDeploymentRequest,
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create deployment (CreateDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint create a dedicated servers deployment in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateDeploymentRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (deployment created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        409: Conflict - ResponseError (deployment already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeployment.create(
        body=body,
        deployment=deployment,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateDeployment)
async def create_deployment_async(
    body: ModelsCreateDeploymentRequest,
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create deployment (CreateDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint create a dedicated servers deployment in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateDeploymentRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (deployment created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        409: Conflict - ResponseError (deployment already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeployment.create(
        body=body,
        deployment=deployment,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateDeploymentClient)
def create_deployment_client(
    body: ModelsCreateDeploymentRequest,
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create deployment for client (CreateDeploymentClient)

    Required permission: NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint create a dedicated servers deployment in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/configs/deployments/{deployment}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateDeploymentRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (deployment created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        409: Conflict - ResponseError (deployment already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeploymentClient.create(
        body=body,
        deployment=deployment,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateDeploymentClient)
async def create_deployment_client_async(
    body: ModelsCreateDeploymentRequest,
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create deployment for client (CreateDeploymentClient)

    Required permission: NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint create a dedicated servers deployment in a namespace.

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/configs/deployments/{deployment}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateDeploymentRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (deployment created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        409: Conflict - ResponseError (deployment already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeploymentClient.create(
        body=body,
        deployment=deployment,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateDeploymentOverride)
def create_deployment_override(
    body: ModelsCreateDeploymentOverrideRequest,
    deployment: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create deployment override (CreateDeploymentOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint create a dedicated servers deployment override in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/version/{version}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateDeploymentOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (deployment override created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        409: Conflict - ResponseError (deployment override already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeploymentOverride.create(
        body=body,
        deployment=deployment,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateDeploymentOverride)
async def create_deployment_override_async(
    body: ModelsCreateDeploymentOverrideRequest,
    deployment: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create deployment override (CreateDeploymentOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint create a dedicated servers deployment override in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/version/{version}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateDeploymentOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (deployment override created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        409: Conflict - ResponseError (deployment override already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeploymentOverride.create(
        body=body,
        deployment=deployment,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateOverrideRegionOverride)
def create_override_region_override(
    body: ModelsCreateRegionOverrideRequest,
    deployment: str,
    region: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create region override for deployment override (CreateOverrideRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint creates a dedicated servers deployment override in a namespace in a region for deployment overrides.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}/regions/{region}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (region override created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        409: Conflict - ResponseError (deployment already has region override)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateOverrideRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateOverrideRegionOverride)
async def create_override_region_override_async(
    body: ModelsCreateRegionOverrideRequest,
    deployment: str,
    region: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create region override for deployment override (CreateOverrideRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint creates a dedicated servers deployment override in a namespace in a region for deployment overrides.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}/regions/{region}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (region override created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        409: Conflict - ResponseError (deployment already has region override)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateOverrideRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateRootRegionOverride)
def create_root_region_override(
    body: ModelsCreateRegionOverrideRequest,
    deployment: str,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create region override (CreateRootRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint creates a dedicated servers deployment override in a namespace in a region for root deployment.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/regions/{region}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (region override created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        409: Conflict - ResponseError (deployment already has region override)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateRootRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateRootRegionOverride)
async def create_root_region_override_async(
    body: ModelsCreateRegionOverrideRequest,
    deployment: str,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create region override (CreateRootRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required scope: social

    This endpoint creates a dedicated servers deployment override in a namespace in a region for root deployment.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/regions/{region}

        method: POST

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        201: Created - ModelsDeploymentWithOverride (region override created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        409: Conflict - ResponseError (deployment already has region override)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateRootRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteDeployment)
def delete_deployment(
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment (DeleteDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment in a namespace

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeployment.create(
        deployment=deployment,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteDeployment)
async def delete_deployment_async(
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment (DeleteDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment in a namespace

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeployment.create(
        deployment=deployment,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteDeploymentClient)
def delete_deployment_client(
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment for client (DeleteDeploymentClient)

    Required permission: NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment in a namespace

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/configs/deployments/{deployment}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeploymentClient.create(
        deployment=deployment,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteDeploymentClient)
async def delete_deployment_client_async(
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment for client (DeleteDeploymentClient)

    Required permission: NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment in a namespace

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/configs/deployments/{deployment}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeploymentClient.create(
        deployment=deployment,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteDeploymentOverride)
def delete_deployment_override(
    deployment: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment Override (DeleteDeploymentOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment override in a namespace

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment override deleted)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeploymentOverride.create(
        deployment=deployment,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteDeploymentOverride)
async def delete_deployment_override_async(
    deployment: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment Override (DeleteDeploymentOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment override in a namespace

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment override deleted)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeploymentOverride.create(
        deployment=deployment,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteOverrideRegionOverride)
def delete_override_region_override(
    deployment: str,
    region: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete region override for deployment override (DeleteOverrideRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment override in a namespace in a region for deployment overrides

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}/regions/{region}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override deleted)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment  not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteOverrideRegionOverride.create(
        deployment=deployment,
        region=region,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteOverrideRegionOverride)
async def delete_override_region_override_async(
    deployment: str,
    region: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete region override for deployment override (DeleteOverrideRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment override in a namespace in a region for deployment overrides

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}/regions/{region}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override deleted)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment  not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteOverrideRegionOverride.create(
        deployment=deployment,
        region=region,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteRootRegionOverride)
def delete_root_region_override(
    deployment: str,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Region Override (DeleteRootRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment override in a namespace in a region for root deployment

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/regions/{region}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override deleted)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteRootRegionOverride.create(
        deployment=deployment,
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteRootRegionOverride)
async def delete_root_region_override_async(
    deployment: str,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Region Override (DeleteRootRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required scope: social

    This endpoint delete a dedicated server deployment override in a namespace in a region for root deployment

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/regions/{region}

        method: DELETE

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override deleted)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteRootRegionOverride.create(
        deployment=deployment,
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAllDeployment)
def get_all_deployment(
    count: int,
    offset: int,
    name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Deployments (GetAllDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required scope: social

    This endpoint get a all deployments in a namespace

    Parameter Offset and Count is Required

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments

        method: GET

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        name: (name) OPTIONAL str in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListDeploymentResponse (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAllDeployment.create(
        count=count,
        offset=offset,
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAllDeployment)
async def get_all_deployment_async(
    count: int,
    offset: int,
    name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Deployments (GetAllDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required scope: social

    This endpoint get a all deployments in a namespace

    Parameter Offset and Count is Required

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments

        method: GET

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        name: (name) OPTIONAL str in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListDeploymentResponse (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAllDeployment.create(
        count=count,
        offset=offset,
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAllDeploymentClient)
def get_all_deployment_client(
    count: int,
    offset: int,
    name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Deployments for client (GetAllDeploymentClient)

    Required permission: NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required scope: social

    This endpoint get a all deployments in a namespace

    Parameter Offset and Count is Required

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/configs/deployments

        method: GET

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        name: (name) OPTIONAL str in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListDeploymentResponse (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAllDeploymentClient.create(
        count=count,
        offset=offset,
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAllDeploymentClient)
async def get_all_deployment_client_async(
    count: int,
    offset: int,
    name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get All Deployments for client (GetAllDeploymentClient)

    Required permission: NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required scope: social

    This endpoint get a all deployments in a namespace

    Parameter Offset and Count is Required

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/configs/deployments

        method: GET

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        name: (name) OPTIONAL str in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListDeploymentResponse (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAllDeploymentClient.create(
        count=count,
        offset=offset,
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetDeployment)
def get_deployment(
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Deployment (GetDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required scope: social

    This endpoint get a dedicated server deployment in a namespace

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: GET

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetDeployment.create(
        deployment=deployment,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetDeployment)
async def get_deployment_async(
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Deployment (GetDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required scope: social

    This endpoint get a dedicated server deployment in a namespace

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: GET

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (ok)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetDeployment.create(
        deployment=deployment,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateDeployment)
def update_deployment(
    body: ModelsUpdateDeploymentRequest,
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update deployment (UpdateDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateDeploymentRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (config not found)

        422: Unprocessable Entity - ResponseError (invalid game version)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateDeployment.create(
        body=body,
        deployment=deployment,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateDeployment)
async def update_deployment_async(
    body: ModelsUpdateDeploymentRequest,
    deployment: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update deployment (UpdateDeployment)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateDeploymentRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (config not found)

        422: Unprocessable Entity - ResponseError (invalid game version)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateDeployment.create(
        body=body,
        deployment=deployment,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateDeploymentOverride)
def update_deployment_override(
    body: ModelsUpdateDeploymentOverrideRequest,
    deployment: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update deployment override (UpdateDeploymentOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment override in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateDeploymentOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment override updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateDeploymentOverride.create(
        body=body,
        deployment=deployment,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateDeploymentOverride)
async def update_deployment_override_async(
    body: ModelsUpdateDeploymentOverrideRequest,
    deployment: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update deployment override (UpdateDeploymentOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment override in a namespace.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateDeploymentOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment override updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateDeploymentOverride.create(
        body=body,
        deployment=deployment,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateOverrideRegionOverride)
def update_override_region_override(
    body: ModelsUpdateRegionOverrideRequest,
    deployment: str,
    region: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update region override for deployment override (UpdateOverrideRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment override in a namespace in a region for deployment overrides.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}/regions/{region}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateOverrideRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateOverrideRegionOverride)
async def update_override_region_override_async(
    body: ModelsUpdateRegionOverrideRequest,
    deployment: str,
    region: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update region override for deployment override (UpdateOverrideRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment override in a namespace in a region for deployment overrides.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/versions/{version}/regions/{region}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

        version: (version) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateOverrideRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateRootRegionOverride)
def update_root_region_override(
    body: ModelsUpdateRegionOverrideRequest,
    deployment: str,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update region override (UpdateRootRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment override in a namespace in a region for root deployment.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/regions/{region}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateRootRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateRootRegionOverride)
async def update_root_region_override_async(
    body: ModelsUpdateRegionOverrideRequest,
    deployment: str,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update region override (UpdateRootRegionOverride)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required scope: social

    This endpoint update a dedicated servers deployment override in a namespace in a region for root deployment.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:CONFIG [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/configs/deployments/{deployment}/overrides/regions/{region}

        method: PATCH

        tags: ["Deployment Config"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUpdateRegionOverrideRequest in body

        deployment: (deployment) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        200: OK - ModelsDeploymentWithOverride (deployment region override updated)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (deployment not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateRootRegionOverride.create(
        body=body,
        deployment=deployment,
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )
